==> lmtpd-0.9.19.tar.gz: lmtpd-0.9.19/readline.c <==
/*
   Copyright 2000, 2001, 2002, 2003 Laurent Wacrenier

   This file is part of PLL lmtpd.

   PLL lmtpd is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   PLL lmtpd is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with PLL lmtpd; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

static char const rcsid[] =
"$Id: readline.c,v 1.23 2003/10/08 17:32:10 lwa Exp $";

#include "config.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <strings.h>
#include <stdarg.h>

#include "lmtp.h"

#define BUFLEN 1024


#if !HAVE_VASPRINTF
/* from http://www.unixpapa.com/incnote/stdio.html */
int vasprintf(char **ret, const char *format, va_list ap)
{
  int len= 100;        /* First guess at the size */
  if ((*ret= (char *)malloc(len)) == NULL) return -1;
  while (1) {
      int nchar= vsnprintf(*ret, len, format, ap);
      if (nchar > -1 && nchar < len) return nchar;
      if (nchar > -1)
	len= nchar+1;
      else
	len*= 2;
      if ((*ret= (char *)realloc(*ret, len)) == NULL)  return -1;
  }
}
#endif

struct headers *setheader(int type, char *fmt, ...) {
  struct headers *h;
  char *header=NULL;
  va_list ap;
  char *dd;
  int len;

  va_start(ap, fmt);
  if ((len=vasprintf(&header, fmt, ap))==-1) {
    return NULL;
  }

  if ((dd=strchr(header, ':'))==NULL) {
    free(header);
    return NULL;
  }

  if ((h = malloc(sizeof(struct headers))) == NULL) {
    free(header);
    return NULL;
  }

  while(header[len-1] == '\n') header[--len] = 0;

  h->header = header;
  h->utf8   = NULL;
  h->dd_len = dd-header;
  h->len    = len;
  h->trace  = 0;
  h->type   = type;
  h->inset  = 0;

  va_end(ap);
  return h;
}

/*
 *  returns
 *    - the header (ended by \0 but not \n\0)
 *      * if not header, dd_len = 0 
 *      * if header, dd_len is the position of the first ':' character
 *    - "\n" normal end of headers
 *    - ""   end of message
 *  return value can be freed by free()
 *  len is the lenght of the malloced string.
 */

char *readheader(FILE *f, int *len, int *dd_len) {
  char *buf=NULL;
  size_t allocated=0;
  int begindot=0;
  *dd_len=*len=0;

  while(1) {
    int c=getc(f), c1;
    switch(c) {
    case '.':
      if (*len==0) {
	begindot=1;
	continue;
      }
      break;
    case ':':
      if (!*dd_len)
	*dd_len=*len;
      break;
    case EOF:
      free(buf);
      *len=0;
      return NULL;
    case '\r':
      if ((c1=getc(f))!='\n') {
	ungetc(c1, f);
	break;
      }
      c=c1;
      /* no break, direct to \n case */
    case '\n':
      if (*len==0) {
	if (begindot)
	  return strdup("");
	else {
	  *len=1;
	  return strdup("\n");
	}
      }
      c1=getc(f);
      ungetc(c1, f);
      if (c1!=' ' && c1 != '\t') {
	buf[(*len)]='\0';
	return realloc(buf, (*len)+1); /* shrink the buffer */
      }
      break;
    }
    if (*len>=allocated) {
      char *tmp;
      allocated+=BUFLEN;
      tmp=(char *)realloc(buf, allocated+sizeof("\n"));
      if (tmp==NULL) {
	free(buf);
	return NULL;
      }
      buf=tmp;
    }
    buf[(*len)++]=c;
  }
}

==> lmtpd-0.10.4.tar.gz: lmtpd-0.10.4/readline.c <==
/*
   Copyright 2000, 2001, 2002, 2003 Laurent Wacrenier

   This file is part of PLL lmtpd.

   PLL lmtpd is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   PLL lmtpd is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with PLL lmtpd; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "config.h"

static char const rcsid[] UNUSED =
"$Id: readline.c,v 1.24 2004/10/26 09:59:20 lwa Exp $";



#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <strings.h>
#include <stdarg.h>

#include "lmtp.h"

#define BUFLEN 1024


#if !HAVE_VASPRINTF
/* from http://www.unixpapa.com/incnote/stdio.html */
int vasprintf(char **ret, const char *format, va_list ap)
{
  int len= 100;        /* First guess at the size */
  if ((*ret= (char *)malloc(len)) == NULL) return -1;
  while (1) {
      int nchar= vsnprintf(*ret, len, format, ap);
      if (nchar > -1 && nchar < len) return nchar;
      if (nchar > -1)
	len= nchar+1;
      else
	len*= 2;
      if ((*ret= (char *)realloc(*ret, len)) == NULL)  return -1;
  }
}
#endif

struct headers *setheader(int type, char *fmt, ...) {
  struct headers *h;
  char *header=NULL;
  va_list ap;
  char *dd;
  int len;

  va_start(ap, fmt);
  if ((len=vasprintf(&header, fmt, ap))==-1) {
    return NULL;
  }

  if ((dd=strchr(header, ':'))==NULL) {
    free(header);
    return NULL;
  }

  if ((h = malloc(sizeof(struct headers))) == NULL) {
    free(header);
    return NULL;
  }

  while(header[len-1] == '\n') header[--len] = 0;

  h->header = header;
  h->utf8   = NULL;
  h->dd_len = dd-header;
  h->len    = len;
  h->trace  = 0;
  h->type   = type;
  h->inset  = 0;

  va_end(ap);
  return h;
}

/*
 *  returns
 *    - the header (ended by \0 but not \n\0)
 *      * if not header, dd_len = 0 
 *      * if header, dd_len is the position of the first ':' character
 *    - "\n" normal end of headers
 *    - ""   end of message
 *  return value can be freed by free()
 *  len is the lenght of the malloced string.
 */

char *readheader(FILE *f, int *len, int *dd_len) {
  char *buf=NULL;
  size_t allocated=0;
  int begindot=0;
  *dd_len=*len=0;

  while(1) {
    int c=getc(f), c1;
    switch(c) {
    case '.':
      if (*len==0) {
	begindot=1;
	continue;
      }
      break;
    case ':':
      if (!*dd_len)
	*dd_len=*len;
      break;
    case EOF:
      free(buf);
      *len=0;
      return NULL;
    case '\r':
      if ((c1=getc(f))!='\n') {
	ungetc(c1, f);
	break;
      }
      c=c1;
      /* no break, direct to \n case */
    case '\n':
      if (*len==0) {
	if (begindot)
	  return strdup("");
	else {
	  *len=1;
	  return strdup("\n");
	}
      }
      c1=getc(f);
      ungetc(c1, f);
      if (c1!=' ' && c1 != '\t') {
	buf[(*len)]='\0';
	return realloc(buf, (*len)+1); /* shrink the buffer */
      }
      break;
    }
    if (*len>=allocated) {
      char *tmp;
      allocated+=BUFLEN;
      tmp=(char *)realloc(buf, allocated+sizeof("\n"));
      if (tmp==NULL) {
	free(buf);
	return NULL;
      }
      buf=tmp;
    }
    buf[(*len)++]=c;
  }
}
